import re
from PyDNI.generator import Generator
from PyDNI import (
    verificar_dni,
    verificar_cif,
    verificar_nie,
    verificar_nif
)

gen = Generator()

# DNI
def test_generar_dni_formato():
    dni = gen.generar_dni()
    assert re.match(r"^\d{8}[A-Z]$", dni)


def test_generar_dni_valido():
    dni = gen.generar_dni()
    assert verificar_dni(dni)

# NIE
def test_generar_nie_formato():
    nie = gen.generar_nie()
    assert re.match(r"^[XYZ]\d{7}[A-Z]$", nie)


def test_generar_nie_valido():
    nie = gen.generar_nie()
    assert verificar_nie(nie)

# CIF
def test_generar_cif_formato():
    cif = gen.generar_cif()
    assert re.match(r"^[ABCDEFGHJKLMNPQRSUVW]\d{7}[A-Z0-9]$", cif)


def test_generar_cif_valido():
    cif = gen.generar_cif()
    assert verificar_cif(cif)

# Varios
def test_generar_varios_dni_sin_repetidos():
    docs = gen.generar_varios(100, "DNI")
    assert len(docs) == len(set(docs))  # Sin repetidos
    assert all(verificar_dni(d) for d in docs)


def test_generar_varios_nie_sin_repetidos():
    docs = gen.generar_varios(50, "NIE")
    assert len(docs) == len(set(docs))
    assert all(verificar_nie(d) for d in docs)


def test_generar_varios_cif_sin_repetidos():
    docs = gen.generar_varios(50, "CIF")
    assert len(docs) == len(set(docs))
    assert all(verificar_cif(d) for d in docs)


def test_generar_varios_auto_varios_tipos():
    docs = gen.generar_varios(100, "AUTO")
    assert len(docs) == len(set(docs))

    # Verifica que al menos hay 2 tipos diferentes
    tipos_detectados = {
        "DNI" if re.match(r"^\d{8}[A-Z]$", d) else
        "NIE" if re.match(r"^[XYZ]\d{7}[A-Z]$", d) else
        "CIF"
        for d in docs
    }
    assert len(tipos_detectados) >= 2



from PyDNI import Generator

def test_persona_tiene_telefono_y_direccion():
    gen = Generator()
    persona = gen.generar_persona()

    assert "telefono" in persona
    assert len(persona["telefono"]) == 9




from PyDNI.phone import PhoneGenerator

def test_generar_movil():
    gen = PhoneGenerator()
    tel = gen.generar_movil()
    assert tel[0] in "67"
    assert len(tel) == 9
    assert tel.isdigit()

def test_generar_fijo():
    gen = PhoneGenerator()
    tel = gen.generar_fijo()
    assert tel[0] in "89"
    assert len(tel) == 9
    assert tel.isdigit()

def test_generar_telefono_auto():
    gen = PhoneGenerator()
    tel = gen.generar_telefono()
    assert len(tel) == 9



# nie.py

def verificar_nie(nie: str) -> bool:
    """
    Verifica si un NIE es válido.
    Formato: Letra inicial (X/Y/Z) + 7 dígitos + letra de control.
    """
    if not isinstance(nie, str):
        return False
    nie = nie.upper().strip()
    if len(nie) != 9:
        return False
    if nie[0] not in "XYZ":
        return False
    if not nie[1:-1].isdigit():
        return False

    # Convertir la letra inicial en número
    prefijos = {"X": "0", "Y": "1", "Z": "2"}
    numero = int(prefijos[nie[0]] + nie[1:-1])

    letras = "TRWAGMYFPDXBNJZSQVHLCKE"
    letra_correcta = letras[numero % 23]
    return nie[-1] == letra_correcta



# test_nif.py

import pytest
from PyDNI import verificar_nif

def test_nif_dni_valido():
    assert verificar_nif("12345678Z") == "DNI válido"

def test_nif_nie_valido():
    assert verificar_nif("X1234567L") == "NIE válido"

def test_nif_cif_valido():
    assert verificar_nif("A58818501") == "CIF válido"

def test_nif_invalido():
    assert verificar_nif("00000000A") == "DNI no válido"



from PyDNI.birthdates import BirthDateGenerator
from datetime import date


def calcular_edad(nacimiento: date) -> int:
    hoy = date.today()
    edad = hoy.year - nacimiento.year
    if (hoy.month, hoy.day) < (nacimiento.month, nacimiento.day):
        edad -= 1
    return edad


def test_menor_de_edad():
    gen = BirthDateGenerator()
    fecha = gen.menor_de_edad()
    assert calcular_edad(fecha) < 18


def test_mayor_de_edad():
    gen = BirthDateGenerator()
    fecha = gen.mayor_de_edad()
    assert calcular_edad(fecha) >= 18


def test_generar_menor():
    gen = BirthDateGenerator()
    fecha = gen.generar("menor")
    assert calcular_edad(fecha) < 18


def test_generar_mayor():
    gen = BirthDateGenerator()
    fecha = gen.generar("mayor")
    assert calcular_edad(fecha) >= 18


def test_generar_aleatorio():
    gen = BirthDateGenerator()
    fecha = gen.generar("aleatorio")
    assert isinstance(fecha, date)


def test_rango_personalizado():
    gen = BirthDateGenerator()
    fecha = gen.rango(edad_min=20, edad_max=30)
    edad = calcular_edad(fecha)
    assert 20 <= edad <= 30


def test_tipo_invalido():
    gen = BirthDateGenerator()
    try:
        gen.generar("inexistente")
        assert False, "Debe lanzar ValueError"
    except ValueError:
        assert True


from PyDNI.emails import EmailGenerator
from PyDNI.generator import Generator


def test_email_aleatorio_basico():
    eg = EmailGenerator()
    email = eg.generar_email_aleatorio()

    assert "@" in email
    usuario, dominio = email.split("@")

    assert len(usuario) >= 8
    assert dominio in EmailGenerator.DOMINIOS_POR_DEFECTO


def test_email_aleatorio_con_dominio_personalizado():
    eg = EmailGenerator()
    email = eg.generar_email_aleatorio("midominio.com")

    assert email.endswith("@midominio.com")


def test_email_nombre_simple():
    eg = EmailGenerator()
    email = eg.generar_email_nombre("Carlos Ruiz Gómez", "empresa.com")

    assert email == "cruiz@empresa.com"


def test_email_nombre_con_acentos_y_mayusculas():
    eg = EmailGenerator()
    email = eg.generar_email_nombre("María López García", "test.com")

    # María → m ; López → lopez
    assert email == "mlopez@test.com"


def test_email_nombre_compuesto():
    eg = EmailGenerator()
    email = eg.generar_email_nombre("Ana María Pérez López", "test.com")

    assert email == "aperez@test.com"


def test_email_generico_con_nombre():
    eg = EmailGenerator()
    email = eg.generar_email("Luis García Fernández")

    usuario, dominio = email.split("@")
    assert usuario == "lgarcia"
    assert dominio in EmailGenerator.DOMINIOS_POR_DEFECTO


def test_email_generico_sin_nombre():
    eg = EmailGenerator()
    email = eg.generar_email()

    assert "@" in email
    assert len(email.split("@")[0]) >= 8


def test_error_nombre_incompleto():
    eg = EmailGenerator()
    try:
        eg.generar_email_nombre("Carlos Ruiz", "dom.com")
        assert False, "Debe lanzar ValueError por nombre incompleto"
    except ValueError:
        assert True



# test_nie.py

import pytest
from PyDNI import verificar_nie

def test_nie_valido():
    assert verificar_nie("X1234567L") is True

def test_nie_invalido_letra_control():
    assert verificar_nie("X1234567A") is False

def test_nie_invalido_formato():
    assert verificar_nie("Y12L") is False



from .dni import verificar_dni
from .cif import verificar_cif
from .nie import verificar_nie
from .nif import verificar_nif
from .utils import verificar_identificador
from .generator import Generator
from .emails import EmailGenerator
from .birthdates import BirthDateGenerator
from .phone import PhoneGenerator
from .faker_provider import PyDNIFakerProvider

__all__ = [
    "verificar_dni",
    "verificar_cif",
    "verificar_nie",
    "verificar_nif",
    "verificar_identificador",
    "Generator",
    "EmailGenerator",
    "BirthDateGenerator",
    "PhoneGenerator",
    "PyDNIFakerProvider",
]

__version__ = "0.9.0"



from faker import Faker
from PyDNI import PyDNIFakerProvider

def test_faker_dni():
    fake = Faker("es_ES")
    fake.add_provider(PyDNIFakerProvider)

    dni = fake.dni()
    assert isinstance(dni, str)
    assert len(dni) == 9

def test_faker_nie():
    fake = Faker("es_ES")
    fake.add_provider(PyDNIFakerProvider)

    nie = fake.nie()
    assert isinstance(nie, str)

def test_faker_persona():
    fake = Faker("es_ES")
    fake.add_provider(PyDNIFakerProvider)

    persona = fake.persona()

    assert "nombre" in persona
    assert "documento" in persona
    assert "email" in persona
    assert "telefono" in persona
    assert "fecha_nacimiento" in persona

def test_faker_email_persona():
    fake = Faker("es_ES")
    fake.add_provider(PyDNIFakerProvider)

    email = fake.email_persona("empresa.com")
    assert email.endswith("@empresa.com")

def test_faker_telefono():
    fake = Faker("es_ES")
    fake.add_provider(PyDNIFakerProvider)

    tel = fake.telefono()
    assert tel.isdigit()



# generator.py

import random
from .dni import verificar_dni
from .cif import verificar_cif
from .emails import EmailGenerator
from .birthdates import BirthDateGenerator
from .phone import PhoneGenerator


LETRAS_DNI = "TRWAGMYFPDXBNJZSQVHLCKE"
LETRAS_CIF = "ABCDEFGHJKLMNPQRSUVW"


class Generator:
    """Generador de DNIs, NIEs, CIFs, NIF y nombres para pruebas."""

    NOMBRES_MASCULINOS = ['ANTONIO', 'MANUEL', 'JOSE', 'FRANCISCO', 'DAVID', 'JAVIER', 'DANIEL', 'JUAN', 'JOSE ANTONIO', 'FRANCISCO JAVIER', 'JOSE LUIS', 'CARLOS', 'ALEJANDRO', 'JESUS', 'JOSE MANUEL', 'MIGUEL', 'MIGUEL ANGEL', 'PABLO', 'RAFAEL', 'SERGIO', 'ANGEL', 'PEDRO', 'JORGE', 'FERNANDO', 'JOSE MARIA', 'ALBERTO', 'LUIS', 'ALVARO', 'ADRIAN', 'JUAN CARLOS', 'DIEGO', 'JUAN JOSE', 'RAUL', 'IVAN', 'RUBEN', 'JUAN ANTONIO', 'OSCAR', 'ENRIQUE', 'JUAN MANUEL', 'MARIO', 'SANTIAGO', 'ANDRES', 'RAMON', 'VICTOR', 'VICENTE', 'JOAQUIN', 'EDUARDO', 'HUGO', 'MARCOS', 'ROBERTO', 'JAIME', 'FRANCISCO JOSE', 'IGNACIO', 'JORDI', 'MOHAMED', 'ALFONSO', 'RICARDO', 'MARTIN', 'MARC', 'SALVADOR', 'GABRIEL', 'GUILLERMO', 'GONZALO', 'EMILIO', 'JOSE MIGUEL', 'NICOLAS', 'LUCAS', 'JULIO', 'JULIAN', 'TOMAS', 'SAMUEL', 'AGUSTIN', 'ISMAEL', 'CRISTIAN', 'JOSE RAMON', 'JOAN', 'AITOR', 'HECTOR', 'ALEX', 'MATEO', 'FELIX', 'IKER', 'JUAN FRANCISCO', 'JOSE CARLOS', 'SEBASTIAN', 'RODRIGO', 'CESAR', 'JOSEP', 'JOSE ANGEL', 'ALFREDO', 'VICTOR MANUEL', 'MARIANO', 'JOSE IGNACIO', 'DOMINGO', 'FELIPE', 'LUIS MIGUEL', 'PAU', 'MOHAMMED', 'IZAN', 'XAVIER', 'JOSE', 'ANTONIO', 'JUAN', 'MANUEL', 'FRANCISCO', 'LUIS', 'JAVIER', 'MIGUEL', 'CARLOS', 'ANGEL', 'JESUS', 'DAVID', 'DANIEL', 'ALEJANDRO', 'PEDRO', 'ALBERTO', 'PABLO', 'MARIA', 'FERNANDO', 'RAFAEL', 'JORGE', 'RAMON', 'SERGIO', 'ANDRES', 'DIEGO', 'ENRIQUE', 'ADRIAN', 'VICTOR', 'ALVARO', 'VICENTE', 'IGNACIO', 'RAUL', 'EDUARDO', 'IVAN', 'OSCAR', 'RUBEN', 'SANTIAGO', 'JOAQUIN', 'MARIO', 'GABRIEL', 'ROBERTO', 'MARCOS', 'ALFONSO', 'JAIME', 'HUGO', 'RICARDO', 'MARTIN', 'JULIO', 'MOHAMED', 'EMILIO', 'GUILLERMO', 'NICOLAS', 'SALVADOR', 'TOMAS', 'GONZALO', 'JULIAN', 'JORDI', 'CRISTIAN', 'CESAR', 'MARC', 'AGUSTIN', 'SEBASTIAN', 'LUCAS', 'SAMUEL', 'FELIX', 'JOAN', 'JOSEP', 'HECTOR', 'FELIPE', 'ISMAEL', 'ALFREDO', 'ALEXANDER', 'MATEO', 'ALEX', 'DOMINGO', 'RODRIGO', 'AITOR', 'IKER', 'MARIANO', 'MARCO', 'ESTEBAN', 'XAVIER', 'ARTURO', 'DARIO', 'JOEL', 'GREGORIO', 'AARON', 'LORENZO', 'MOHAMMED', 'ISAAC', 'BORJA', 'ALBERT', 'ERIC', 'JONATHAN', 'OMAR', 'CHRISTIAN', 'BRUNO', 'PAU', 'IZAN', 'CRISTOBAL']
    NOMBRES_FEMENINOS = ['MARIA CARMEN', 'MARIA', 'CARMEN', 'ANA MARIA', 'LAURA', 'MARIA PILAR', 'MARIA DOLORES', 'ISABEL', 'ANA', 'MARIA TERESA', 'JOSEFA', 'MARTA', 'CRISTINA', 'LUCIA', 'MARIA ANGELES', 'MARIA JOSE', 'MARIA ISABEL', 'FRANCISCA', 'ANTONIA', 'SARA', 'PAULA', 'DOLORES', 'ELENA', 'MARIA LUISA', 'RAQUEL', 'ROSA MARIA', 'MANUELA', 'MARIA JESUS', 'JULIA', 'PILAR', 'CONCEPCION', 'ALBA', 'MERCEDES', 'BEATRIZ', 'SILVIA', 'NURIA', 'IRENE', 'PATRICIA', 'ROCIO', 'ANDREA', 'ROSARIO', 'MONTSERRAT', 'JUANA', 'MONICA', 'TERESA', 'ENCARNACION', 'ALICIA', 'MARIA MAR', 'MARINA', 'SANDRA', 'SONIA', 'NATALIA', 'SOFIA', 'SUSANA', 'ANGELA', 'YOLANDA', 'CLAUDIA', 'ROSA', 'CARLA', 'EVA', 'MARGARITA', 'MARIA JOSEFA', 'INMACULADA', 'ANA ISABEL', 'MARIA MERCEDES', 'MARIA ROSARIO', 'NOELIA', 'DANIELA', 'ESTHER', 'VERONICA', 'CAROLINA', 'MARTINA', 'NEREA', 'INES', 'MIRIAM', 'EVA MARIA', 'MARIA VICTORIA', 'LORENA', 'MARIA ELENA', 'ANA BELEN', 'VICTORIA', 'MARIA ROSA', 'ALEJANDRA', 'ANGELES', 'MARIA CONCEPCION', 'CELIA', 'LIDIA', 'FATIMA', 'MARIA ANTONIA', 'AMPARO', 'AINHOA', 'OLGA', 'CATALINA', 'MARIA NIEVES', 'CLARA', 'ADRIANA', 'VALERIA', 'ANNA', 'MARIA CRISTINA', 'EMMA', 'MARIA', 'CARMEN', 'ANA', 'ISABEL', 'DOLORES', 'PILAR', 'TERESA', 'ROSA', 'JOSEFA', 'CRISTINA', 'LAURA', 'ANGELES', 'ELENA', 'LUCIA', 'ANTONIA', 'MARTA', 'FRANCISCA', 'MERCEDES', 'LUISA', 'PAULA', 'JOSE', 'ROSARIO', 'CONCEPCION', 'SARA', 'RAQUEL', 'PATRICIA', 'ROCIO', 'EVA', 'BEATRIZ', 'ANDREA', 'VICTORIA', 'JULIA', 'MANUELA', 'JESUS', 'JUANA', 'BELEN', 'ALBA', 'SILVIA', 'ESTHER', 'SOFIA', 'IRENE', 'NURIA', 'SANDRA', 'ANGELA', 'MONICA', 'MONTSERRAT', 'ENCARNACION', 'ALICIA', 'MARINA', 'INMACULADA', 'MAR', 'SONIA', 'YOLANDA', 'CLAUDIA', 'NATALIA', 'CAROLINA', 'SUSANA', 'MARGARITA', 'ALEJANDRA', 'DANIELA', 'INES', 'CARLA', 'VERONICA', 'GLORIA', 'LUZ', 'LOURDES', 'AMPARO', 'LORENA', 'FATIMA', 'NIEVES', 'SOLEDAD', 'NOELIA', 'BEGOÑA', 'BLANCA', 'OLGA', 'MARTINA', 'MIRIAM', 'NEREA', 'CLARA', 'ADRIANA', 'MILAGROS', 'LIDIA', 'ESPERANZA', 'ANNA', 'CONSUELO', 'ASUNCION', 'CATALINA', 'CELIA', 'VALENTINA', 'VALERIA', 'DIANA', 'AURORA', 'GABRIELA', 'MAGDALENA', 'ALEXANDRA', 'ELIZABETH', 'VANESA', 'ELISA', 'AINHOA', 'EMILIA']
    APELLIDOS = ['GARCIA', 'RODRIGUEZ', 'GONZALEZ', 'FERNANDEZ', 'LOPEZ', 'MARTINEZ', 'SANCHEZ', 'PEREZ', 'GOMEZ', 'MARTIN', 'JIMENEZ', 'HERNANDEZ', 'RUIZ', 'DIAZ', 'MORENO', 'MUÑOZ', 'ALVAREZ', 'ROMERO', 'GUTIERREZ', 'ALONSO', 'TORRES', 'NAVARRO', 'DOMINGUEZ', 'RAMIREZ', 'RAMOS', 'VAZQUEZ', 'GIL', 'SERRANO', 'MORALES', 'MOLINA', 'SUAREZ', 'CASTRO', 'BLANCO', 'DELGADO', 'ORTEGA', 'ORTIZ', 'MARIN', 'RUBIO', 'MEDINA', 'NUÑEZ', 'CASTILLO', 'SANZ', 'CORTES', 'IGLESIAS', 'SANTOS', 'GARRIDO', 'GUERRERO', 'LOZANO', 'FLORES', 'CANO', 'CRUZ', 'MENDEZ', 'HERRERA', 'PEÑA', 'PRIETO', 'LEON', 'CABRERA', 'MARQUEZ', 'REYES', 'GALLEGO', 'VIDAL', 'CALVO', 'CAMPOS', 'VEGA', 'FUENTES', 'AGUILAR', 'CARRASCO', 'VARGAS', 'CABALLERO', 'DIEZ', 'NIETO', 'SANTANA', 'GIMENEZ', 'HIDALGO', 'MONTERO', 'ROJAS', 'BENITEZ', 'PASCUAL', 'HERRERO', 'ARIAS', 'SANTIAGO', 'LORENZO', 'DURAN', 'MORA', 'IBAÑEZ', 'FERRER', 'CARMONA', 'VICENTE', 'SOTO', 'ROMAN', 'CRESPO', 'RIVERA', 'PARRA', 'SILVA', 'VELASCO', 'PASTOR', 'BRAVO', 'SAEZ', 'MOYA', 'MENDOZA']

    def __init__(self):
        self.email_gen = EmailGenerator()
        self.birth_gen = BirthDateGenerator()
        self.phone_gen = PhoneGenerator()

    def generar_dni(self) -> str:
        numero = random.randint(0, 99999999)
        letra = LETRAS_DNI[numero % 23]
        return f"{numero:08d}{letra}"

    def generar_nie(self) -> str:
        inicial = random.choice("XYZ")
        numero = random.randint(0, 9999999)
        # Transformación NIE → DNI para cálculo
        mapa = {"X": "0", "Y": "1", "Z": "2"}
        numero_transformado = int(mapa[inicial] + f"{numero:07d}")
        letra = LETRAS_DNI[numero_transformado % 23]
        return f"{inicial}{numero:07d}{letra}"

    def generar_cif(self) -> str:
        inicial = random.choice(LETRAS_CIF)
        numero = f"{random.randint(0, 9999999):07d}"

        # Cálculo control (AEAT)
        suma_pares = sum(int(numero[i]) for i in range(1, 7, 2))
        suma_impares = 0
        for i in range(0, 7, 2):
            doble = int(numero[i]) * 2
            suma_impares += doble if doble < 10 else doble - 9

        total = suma_pares + suma_impares
        digito = (10 - (total % 10)) % 10

        control_letras = "JABCDEFGHI"
        control_letra = control_letras[digito]

        # Reglas según tipo
        if inicial in "PQRSNW":     # Solo letra
            control = control_letra
        elif inicial in "ABEH":     # Solo número
            control = str(digito)
        else:                       # Ambos válidos
            control = random.choice([str(digito), control_letra])

        return f"{inicial}{numero}{control}"

    def generar_varios(self, cantidad: int, tipo: str) -> list:
        """Genera una lista de documentos válidos sin repetir."""
        generados = set()

        while len(generados) < cantidad:
            if tipo == "DNI":
                doc = self.generar_dni()
            elif tipo == "NIE":
                doc = self.generar_nie()
            elif tipo == "CIF":
                doc = self.generar_cif()
            elif tipo == "AUTO":
                # Genera un tipo aleatorio
                doc = random.choice([
                    self.generar_dni(),
                    self.generar_nie(),
                    self.generar_cif()
                ])
            else:
                raise ValueError("Tipo no reconocido (DNI, NIE, CIF, AUTO)")

            generados.add(doc)

        return list(generados)
    
    def generar_nombre(self, sexo: str | None = None) -> str:
        sexo = sexo.lower() if sexo else "aleatorio"

        if sexo == "aleatorio":
            sexo = random.choice(["masculino", "femenino"])

        if sexo not in ["masculino", "femenino"]:
            raise ValueError("El sexo debe ser 'masculino', 'femenino' o 'aleatorio'")

        if sexo == "masculino":
            nombre = random.choice(self.NOMBRES_MASCULINOS)
        else:
            nombre = random.choice(self.NOMBRES_FEMENINOS)

        apellido1, apellido2 = random.sample(self.APELLIDOS, 2)

        return f"{nombre} {apellido1} {apellido2}"

    
    def generar_persona(self, sexo: str | None = None, tipo_doc: str | None = None, edad: str = "aleatorio") -> dict:
        """
        Genera una persona completa con:
        - Nombre completo
        - Sexo (masculino, femenino o aleatorio)
        - Documento válido (DNI, NIE o CIF)
        - Email basado en nombre
        - Fecha de nacimiento (menor, mayor o aleatoria)

        Parámetros:
            sexo: "masculino", "femenino", "aleatorio" o None
            tipo_doc: "DNI", "NIE", "CIF", "AUTO" o None
            edad: "menor", "mayor", "aleatorio"
        """

        # Resolver sexo
        if sexo is None or sexo.lower() == "aleatorio":
            sexo_resuelto = random.choice(["masculino", "femenino"])
        elif sexo.lower() in ("masculino", "femenino"):
            sexo_resuelto = sexo.lower()
        else:
            raise ValueError(f"Sexo desconocido: {sexo}")

        # Nombre
        nombre = self.generar_nombre(sexo_resuelto)

        # Documento
        if (
            tipo_doc is None
            or tipo_doc.lower() == "aleatorio"
            or tipo_doc.upper() == "AUTO"
        ):
            tipo_doc_resuelto = random.choice(["DNI", "NIE", "CIF"])
        else:
            tipo_doc_resuelto = tipo_doc.upper()

        if tipo_doc_resuelto == "DNI":
            documento = self.generar_dni()
        elif tipo_doc_resuelto == "NIE":
            documento = self.generar_nie()
        elif tipo_doc_resuelto == "CIF":
            documento = self.generar_cif()
        else:
            raise ValueError(f"Tipo de documento desconocido: {tipo_doc}")

        # Email
        email = self.email_gen.generar_email(nombre)

        # Fecha de nacimiento
        fecha_nacimiento = self.birth_gen.generar(edad)

        # Telefono
        telefono = self.phone_gen.generar_telefono()


        return {
            "nombre": nombre,
            "sexo": sexo_resuelto,
            "tipo_documento": tipo_doc_resuelto,
            "documento": documento,
            "email": email,
            "fecha_nacimiento": fecha_nacimiento.isoformat(),
            "telefono": telefono,
        }




from PyDNI.generator import Generator
from PyDNI import (
    verificar_dni,
    verificar_nie,
    verificar_cif,
)

gen = Generator()

def test_generar_persona_estructura():
    persona = gen.generar_persona()
    assert "nombre" in persona
    assert "sexo" in persona
    assert "tipo_documento" in persona
    assert "documento" in persona

def test_generar_persona_nombre_valido():
    persona = gen.generar_persona()
    partes = persona["nombre"].split()

    ap1 = partes[-2]
    ap2 = partes[-1]
    nombre = " ".join(partes[:-2])

    # El primer token del nombre indica el género base
    primer_nombre = nombre.split()[0]

    assert primer_nombre in Generator.NOMBRES_MASCULINOS + Generator.NOMBRES_FEMENINOS
    assert ap1 in Generator.APELLIDOS
    assert ap2 in Generator.APELLIDOS

def test_generar_persona_documento_valido():
    persona = gen.generar_persona()
    tipo = persona["tipo_documento"]
    doc = persona["documento"]

    if tipo == "DNI":
        assert verificar_dni(doc)
    elif tipo == "NIE":
        assert verificar_nie(doc)
    elif tipo == "CIF":
        assert verificar_cif(doc)
    else:
        assert False, f"Tipo de documento inesperado: {tipo}"

def test_generar_persona_masculina():
    persona = gen.generar_persona(sexo="masculino")
    nombre = persona["nombre"].split()[0]

    assert nombre in Generator.NOMBRES_MASCULINOS
    assert persona["sexo"] == "masculino"

def test_generar_persona_femenina():
    persona = gen.generar_persona(sexo="femenino")
    nombre = persona["nombre"].split()[0]

    assert nombre in Generator.NOMBRES_FEMENINOS
    assert persona["sexo"] == "femenino"

def test_generar_persona_con_dni():
    persona = gen.generar_persona(tipo_doc="DNI")
    assert persona["tipo_documento"] == "DNI"
    assert verificar_dni(persona["documento"])

def test_generar_persona_con_nie():
    persona = gen.generar_persona(tipo_doc="NIE")
    assert persona["tipo_documento"] == "NIE"
    assert verificar_nie(persona["documento"])

def test_generar_persona_con_cif():
    persona = gen.generar_persona(tipo_doc="CIF")
    assert persona["tipo_documento"] == "CIF"
    assert verificar_cif(persona["documento"])

def test_generar_persona_sexo_aleatorio():
    sexos = {gen.generar_persona(sexo="aleatorio")["sexo"] for _ in range(20)}
    assert sexos.issubset({"masculino", "femenino"})
    assert len(sexos) >= 1  # debería haber al menos 1 sexo generado

def test_generar_persona_tipo_aleatorio():
    tipos = {gen.generar_persona(tipo_doc="aleatorio")["tipo_documento"] for _ in range(30)}
    assert tipos.issubset({"DNI", "NIE", "CIF"})
    assert len(tipos) >= 1

def test_generar_persona_sexo_invalido():
    try:
        gen.generar_persona(sexo="robot")
        assert False, "Debe lanzar ValueError"
    except ValueError:
        assert True

def test_generar_persona_tipo_documento_invalido():
    try:
        gen.generar_persona(tipo_doc="PASAPORTE")
        assert False, "Debe lanzar ValueError"
    except ValueError:
        assert True



import random

class PhoneGenerator:
    def generar_movil(self) -> str:
        prefijo = random.choice(["6", "7"])
        return prefijo + "".join(str(random.randint(0, 9)) for _ in range(8))

    def generar_fijo(self) -> str:
        prefijo = random.choice(["8", "9"])
        return prefijo + "".join(str(random.randint(0, 9)) for _ in range(8))

    def generar_telefono(self, tipo: str = "AUTO") -> str:
        tipo = tipo.upper()
        if tipo == "MOVIL":
            return self.generar_movil()
        elif tipo == "FIJO":
            return self.generar_fijo()
        elif tipo == "AUTO":
            return random.choice([self.generar_movil, self.generar_fijo])()
        else:
            raise ValueError("Tipo de teléfono inválido")



# dni.py

def verificar_dni(dni: str) -> bool:
    dni = dni.upper().strip()
    if len(dni) != 9 or not dni[:-1].isdigit():
        return False
    letras = "TRWAGMYFPDXBNJZSQVHLCKE"
    numero = int(dni[:-1])
    letra_correcta = letras[numero % 23]
    return dni[-1] == letra_correcta



# test_cif.py

import pytest
from PyDNI import verificar_cif

def test_cif_valido_telefonica():
    # CIF real de Telefónica
    assert verificar_cif("A58818501") is True

def test_cif_valido_ayuntamiento_madrid():
    # CIF real de Ayuntamiento de Madrid (excepción)
    assert verificar_cif("Q2816003A") is True

def test_cif_invalido():
    assert verificar_cif("B12345678") is False



from datetime import date
from PyDNI.generator import Generator


def calcular_edad(nacimiento: date) -> int:
    hoy = date.today()
    edad = hoy.year - nacimiento.year
    if (hoy.month, hoy.day) < (nacimiento.month, nacimiento.day):
        edad -= 1
    return edad

def test_generar_persona_incluye_fecha_nacimiento():
    gen = Generator()
    persona = gen.generar_persona()

    assert "fecha_nacimiento" in persona

    fecha = date.fromisoformat(persona["fecha_nacimiento"])
    assert isinstance(fecha, date)

def test_generar_persona_menor_de_edad():
    gen = Generator()
    persona = gen.generar_persona(edad="menor")

    fecha = date.fromisoformat(persona["fecha_nacimiento"])
    edad = calcular_edad(fecha)

    assert edad < 18

def test_generar_persona_mayor_de_edad():
    gen = Generator()
    persona = gen.generar_persona(edad="mayor")

    fecha = date.fromisoformat(persona["fecha_nacimiento"])
    edad = calcular_edad(fecha)

    assert edad >= 18

def test_generar_persona_edad_aleatoria():
    gen = Generator()
    persona = gen.generar_persona(edad="aleatorio")

    fecha = date.fromisoformat(persona["fecha_nacimiento"])
    assert isinstance(fecha, date)

def test_generar_persona_edad_random_alias():
    gen = Generator()
    persona = gen.generar_persona(edad="random")

    fecha = date.fromisoformat(persona["fecha_nacimiento"])
    assert isinstance(fecha, date)

def test_generar_persona_con_todos_parametros():
    gen = Generator()
    persona = gen.generar_persona(sexo="masculino", tipo_doc="DNI", edad="mayor")

    assert isinstance(persona["nombre"], str)
    assert persona["sexo"] == "masculino"
    assert persona["tipo_documento"] == "DNI"
    assert isinstance(persona["documento"], str)
    assert "@" in persona["email"]

    fecha = date.fromisoformat(persona["fecha_nacimiento"])
    assert calcular_edad(fecha) >= 18

def test_generar_persona_sin_parametros():
    gen = Generator()
    persona = gen.generar_persona()

    assert isinstance(persona["nombre"], str)
    assert persona["sexo"] in ("masculino", "femenino")
    assert persona["tipo_documento"] in ("DNI", "NIE", "CIF")

    fecha = date.fromisoformat(persona["fecha_nacimiento"])
    assert isinstance(fecha, date)

def test_generar_persona_con_edad_invalida():
    gen = Generator()

    try:
        gen.generar_persona(edad="anciano")
        assert False, "Debe lanzar ValueError para edad desconocida"
    except ValueError:
        assert True

def test_generar_persona_sin_fecha_nula():
    gen = Generator()
    persona = gen.generar_persona()

    assert persona["fecha_nacimiento"] is not None



# utils.py

from .dni import verificar_dni
from .cif import verificar_cif
from .nie import verificar_nie

def verificar_identificador(valor: str) -> str:
    valor = valor.strip().upper()
    if len(valor) == 9:
        if valor[0] in "XYZ":
            return "NIE válido" if verificar_nie(valor) else "NIE no válido"
        elif valor[0].isalpha():
            return "CIF válido" if verificar_cif(valor) else "CIF no válido"
        elif valor[-1].isalpha():
            return "DNI válido" if verificar_dni(valor) else "DNI no válido"
    return "Formato no reconocido"




from setuptools import setup, find_packages

setup(
    name="PyDNI",
    version="0.9.0",
    description="Spanish identity utilities: validation and generation of DNI, NIE, CIF, NIF, names, emails and birthdates.",
    long_description=open("README.md", encoding="utf-8").read(),
    long_description_content_type="text/markdown",
    author="Alberto Gonzalez",
    author_email="agonzalezla@protonmail.com",
    url="https://github.com/agonzalezla/PyDNI",
    license="MIT",
    packages=find_packages(),
    python_requires=">=3.7",
    extras_require={
        "faker": ["faker>=20.0"],
        "dev": ["pytest", "pytest-cov"]
    },
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Intended Audience :: Information Technology",
        "Topic :: Software Development :: Libraries",
        "Topic :: Software Development :: Testing",
        "Topic :: Utilities",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Operating System :: OS Independent",
        "Natural Language :: Spanish",
        "Natural Language :: English",
    ]
)


from faker.providers import BaseProvider
from .generator import Generator


class PyDNIFakerProvider(BaseProvider):
    """
    Provider oficial de PyDNI para Faker
    Permite generar personas, DNIs, NIEs, CIFs, teléfonos, emails y fechas
    """

    def __init__(self, generator=None):
        super().__init__(generator)
        self.gen = Generator()

    def dni(self):
        return self.gen.generar_dni()

    def nie(self):
        return self.gen.generar_nie()

    def cif(self):
        return self.gen.generar_cif()

    def documento(self):
        return self.gen.generar_documento()

    def persona(self, sexo=None, tipo_doc=None, edad="aleatorio"):
        return self.gen.generar_persona(sexo=sexo, tipo_doc=tipo_doc, edad=edad)

    def nombre(self, sexo=None):
        return self.gen.generar_nombre(sexo)

    def email_persona(self, dominio=None):
        persona = self.gen.generar_persona()
        if dominio:
            return self.gen.email_gen.generar_email_nombre(
                persona["nombre"], dominio
            )
        return persona["email"]

    def telefono(self, tipo="movil"):
        return self.gen.phone_gen.generar_telefono(tipo)

    def fecha_nacimiento(self, tipo="aleatorio"):
        return self.gen.birth_gen.generar_fecha(tipo)



import random
import string
import unicodedata


class EmailGenerator:
    DOMINIOS_POR_DEFECTO = [
        "gmail.com",
        "hotmail.com",
        "yahoo.es",
        "outlook.com",
        "proton.me",
        "icloud.com"
    ]

    def _limpiar(self, texto: str) -> str:
        """
        Elimina acentos y caracteres especiales.
        """
        return "".join(
            c
            for c in unicodedata.normalize("NFD", texto)
            if unicodedata.category(c) != "Mn"
        ).lower()

    def generar_email_aleatorio(self, dominio: str | None = None) -> str:
        """
        Genera un email aleatorio:
        - usuario aleatorio de 8–12 caracteres
        - dominio aleatorio o indicado por el usuario
        """
        letras = string.ascii_lowercase + string.digits
        usuario = "".join(random.choices(letras, k=random.randint(8, 12)))

        if dominio is None:
            dominio = random.choice(self.DOMINIOS_POR_DEFECTO)

        return f"{usuario}@{dominio}"

    def generar_email_nombre(self, nombre_completo: str, dominio: str | None = None) -> str:
        """
        Genera un email basado en:
        - primera letra del nombre
        - primer apellido
        Ej: 'María Ruiz Gómez' → mruiz@dominio.com
        """
        partes = nombre_completo.split()
        if len(partes) < 3:
            raise ValueError("El nombre completo debe incluir nombre y dos apellidos.")

        nombre = self._limpiar(partes[0])
        apellido1 = self._limpiar(partes[-2])

        usuario = f"{nombre[0]}{apellido1}"

        if dominio is None:
            dominio = random.choice(self.DOMINIOS_POR_DEFECTO)

        return f"{usuario}@{dominio}"

    def generar_email(self, nombre_completo: str | None = None, dominio: str | None = None) -> str:
        """
        Genera un email:
        - Si se pasa nombre completo → email personalizado.
        - Si no → email aleatorio.
        """
        if nombre_completo:
            return self.generar_email_nombre(nombre_completo, dominio)
        else:
            return self.generar_email_aleatorio(dominio)



from PyDNI.generator import Generator
from PyDNI.emails import EmailGenerator


def test_persona_incluye_email():
    gen = Generator()
    persona = gen.generar_persona()

    assert "email" in persona
    assert "@" in persona["email"]


def test_email_persona_formato_correcto():
    gen = Generator()
    persona = gen.generar_persona()

    usuario, dominio = persona["email"].split("@")

    assert len(usuario) >= 2  # mínimo: inicial + apellido
    assert len(dominio) >= 3  # dominio razonable


def test_email_persona_coincide_con_nombre():
    gen = Generator()
    persona = gen.generar_persona()

    nombre = persona["nombre"]
    email = persona["email"]

    partes = nombre.split()
    inicial = partes[0][0].lower()

    eg = EmailGenerator()
    apellido = eg._limpiar(partes[-2])  # ← limpieza necesaria

    assert email.startswith(inicial + apellido)


def test_email_persona_con_nombres_compuestos():
    gen = Generator()
    eg = EmailGenerator()

    compuesto = False

    for _ in range(50):
        persona = gen.generar_persona()
        nombre = persona["nombre"]

        partes = nombre.split()
        if len(partes) > 3:  # nombre compuesto
            compuesto = True

            inicial = partes[0][0].lower()
            apellido = eg._limpiar(partes[-2])  # limpieza igual que el generador

            assert persona["email"].startswith(inicial + apellido)

    assert compuesto, "No se generó ningún nombre compuesto en 50 intentos"



from datetime import date, timedelta
import random


class BirthDateGenerator:
    """
    Genera fechas de nacimiento realistas para pruebas.
    Permite generar:
    - Personas menores de edad (<18)
    - Personas adultas (≥18)
    - Fechas aleatorias sin restricciones
    - Fechas con rango personalizado
    """

    def __init__(self, seed: int | None = None):
        if seed is not None:
            random.seed(seed)

    @staticmethod
    def _random_date_between(start: date, end: date) -> date:
        """Devuelve una fecha aleatoria entre dos fechas."""
        delta = end - start
        dias = random.randint(0, delta.days)
        return start + timedelta(days=dias)


    def menor_de_edad(self) -> date:
        """Devuelve una fecha para alguien con 0-17 años."""
        hoy = date.today()
        fecha_max = hoy  # recién nacido
        fecha_min = hoy.replace(year=hoy.year - 18)  # casi 18 pero aún menor
        return self._random_date_between(fecha_min, fecha_max)

    def mayor_de_edad(self) -> date:
        """Devuelve una fecha para alguien con ≥18 años."""
        hoy = date.today()
        fecha_max = hoy.replace(year=hoy.year - 18)
        fecha_min = hoy.replace(year=hoy.year - 100)
        return self._random_date_between(fecha_min, fecha_max)

    def aleatoria(self) -> date:
        """Devuelve una fecha aleatoria sin requisitos de edad."""
        hoy = date.today()
        fecha_min = hoy.replace(year=hoy.year - 100)
        fecha_max = hoy
        return self._random_date_between(fecha_min, fecha_max)

    def rango(self, edad_min: int, edad_max: int) -> date:
        """Genera una fecha para una edad dentro del rango dado."""
        hoy = date.today()
        fecha_max = hoy.replace(year=hoy.year - edad_min)
        fecha_min = hoy.replace(year=hoy.year - edad_max - 1)
        return self._random_date_between(fecha_min, fecha_max)

    def generar(self, tipo: str = "aleatorio") -> date:
        """
        Genera una fecha según el tipo:
        - 'menor'   → <18
        - 'mayor'   → ≥18
        - 'aleatorio'
        """
        tipo = tipo.lower()

        if tipo == "menor":
            return self.menor_de_edad()

        if tipo == "mayor":
            return self.mayor_de_edad()

        if tipo in ("aleatorio", "random"):
            return self.aleatoria()

        raise ValueError(f"Tipo de fecha desconocido: {tipo}")



from PyDNI.generator import Generator

gen = Generator()

def _extraer_nombre_apellidos(nombre_completo: str):
    """
    Devuelve:
      - nombre (puede tener 1 o varias palabras)
      - apellido1
      - apellido2
    """
    partes = nombre_completo.split()
    ap1 = partes[-2]
    ap2 = partes[-1]
    nombre = " ".join(partes[:-2])
    return nombre, ap1, ap2

def test_generar_nombre_masculino():
    nombre_completo = gen.generar_nombre("masculino")
    nombre, ap1, ap2 = _extraer_nombre_apellidos(nombre_completo)

    primer_nombre = nombre.split()[0]

    assert primer_nombre in Generator.NOMBRES_MASCULINOS
    assert ap1 in Generator.APELLIDOS
    assert ap2 in Generator.APELLIDOS

def test_generar_nombre_femenino():
    nombre_completo = gen.generar_nombre("femenino")
    nombre, ap1, ap2 = _extraer_nombre_apellidos(nombre_completo)

    primer_nombre = nombre.split()[0]

    assert primer_nombre in Generator.NOMBRES_FEMENINOS
    assert ap1 in Generator.APELLIDOS
    assert ap2 in Generator.APELLIDOS

def test_generar_nombre_aleatorio():
    nombre_completo = gen.generar_nombre("aleatorio")
    nombre, ap1, ap2 = _extraer_nombre_apellidos(nombre_completo)

    primer_nombre = nombre.split()[0]

    assert primer_nombre in Generator.NOMBRES_MASCULINOS + Generator.NOMBRES_FEMENINOS
    assert ap1 in Generator.APELLIDOS
    assert ap2 in Generator.APELLIDOS

def test_generar_nombre_sin_parametro_es_aleatorio():
    nombre_completo = gen.generar_nombre()
    nombre, ap1, ap2 = _extraer_nombre_apellidos(nombre_completo)

    primer_nombre = nombre.split()[0]

    assert primer_nombre in Generator.NOMBRES_MASCULINOS + Generator.NOMBRES_FEMENINOS
    assert ap1 in Generator.APELLIDOS
    assert ap2 in Generator.APELLIDOS

def test_generar_nombre_formato_correcto():
    nombre_completo = gen.generar_nombre("masculino")
    # Debe tener mínimo 3 palabras: nombre + 2 apellidos
    assert len(nombre_completo.split()) >= 3

def test_generar_nombre_apellidos_distintos():
    for _ in range(50):
        nombre_completo = gen.generar_nombre("aleatorio")
        _, ap1, ap2 = _extraer_nombre_apellidos(nombre_completo)
        assert ap1 != ap2

def test_generar_nombre_sexo_invalido():
    try:
        gen.generar_nombre("otro")
        assert False, "Debe lanzar ValueError"
    except ValueError:
        assert True